==> embrace-android-sdk-3.3.0-beta1-sources.jar: io/embrace/android/embracesdk/EmbraceActivityService.java <==
package io.embrace.android.embracesdk;

import android.app.Activity;
import android.app.Application;
import android.arch.lifecycle.Lifecycle;
import android.arch.lifecycle.OnLifecycleEvent;
import android.arch.lifecycle.ProcessLifecycleOwner;
import android.content.res.Configuration;
import android.os.Bundle;

import com.fernandocejas.arrow.checks.Preconditions;
import com.fernandocejas.arrow.optional.Optional;

import java.lang.ref.WeakReference;
import java.util.concurrent.CopyOnWriteArrayList;

import io.embrace.android.embracesdk.annotation.StartupActivity;
import java9.util.stream.StreamSupport;

/**
 * Service tracking the app's current activity and background state, and dispatching events to other
 * services as required.
 * <p>
 * See activity lifecycle documentation here:
 * https://developer.android.com/guide/components/activities/activity-lifecycle
 * <p>
 * We only need to track the `onForeground` and `onActivityPaused` lifecycle hooks for background
 * detection, as these are the only two which are consistently fired in all scenarios.
 */
final class EmbraceActivityService implements ActivityService {

    private static final String ERROR_FAILED_TO_NOTIFY = "Failed to notify EmbraceActivityService listener";
    /**
     * This Worker responsibility is to host the delayed execution of check to
     * establish is the app is in background or not.
     */
    private final ScheduledWorker appStateWorker = ScheduledWorker.ofSingleThread("App state service");

    /**
     * The memory service, it's provided on the instantiation of the service.
     */
    private final MemoryService memoryService;

    /**
     * List of listeners that subscribe to activity events.
     */
    private final CopyOnWriteArrayList<ActivityListener> listeners = new CopyOnWriteArrayList<>();

    /**
     * The application.
     */
    private final Application application;

    /**
     * The currently active activity.
     */
    private volatile WeakReference<Activity> currentActivity = new WeakReference<>(null);

    /**
     * States if the activity foreground phase comes from a cold start or not.
     * Checked every time an activity executes a foreground phase.
     */
    private volatile boolean coldStart = true;

    /**
     * States the initialization time of the EmbraceActivityService, inferring it is initialized
     * from the {@link Embrace#start(Application)} method.
     */
    private volatile long startTime;

    /**
     * Describes the application state.
     */
    private volatile boolean isInBackground = false;

    EmbraceActivityService(
            Application application,
            MemoryService memoryService) {

        this.memoryService = Preconditions.checkNotNull(memoryService);
        this.application = Preconditions.checkNotNull(application);
        this.startTime = System.currentTimeMillis();
        application.registerActivityLifecycleCallbacks(this);
        application.getApplicationContext().registerComponentCallbacks(this);
        ProcessLifecycleOwner.get().getLifecycle().addObserver(this);
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {
        updateStateWithActivity(activity);
    }

    @Override
    public void onActivityStarted(Activity activity) {
        updateStateWithActivity(activity);
        StreamSupport.stream(listeners).forEach(listener -> {
            try {
                listener.onView(activity);
            } catch (Exception ex) {
                EmbraceLogger.logDebug(ERROR_FAILED_TO_NOTIFY, ex);
            }
        });
    }

    @Override
    public void onActivityResumed(Activity activity) {
        if (!activity.getClass().isAnnotationPresent(StartupActivity.class)) {
            // If the activity coming to foreground doesn't have the StartupActivity annotation
            // the the SDK will finalize any pending startup moment.
            StreamSupport.stream(listeners).forEach(listener -> {
                try {
                    listener.applicationStartupComplete();
                } catch (Exception ex) {
                    EmbraceLogger.logDebug(ERROR_FAILED_TO_NOTIFY, ex);
                }
            });
        }
    }

    @Override
    public void onActivityPaused(Activity activity) {
    }

    @Override
    public void onActivityStopped(Activity activity) {
    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {
    }

    @Override
    public void onActivityDestroyed(Activity activity) {
    }

    /**
     * This method will update the current activity for further checking.
     *
     * @param activity the activity involved in the state change.
     */
    private synchronized void updateStateWithActivity(Activity activity) {
        this.currentActivity = new WeakReference<>(activity);
    }

    /**
     * This method will be called by the ProcessLifecycleOwner when the main app process calls
     * ON START.
     **/
    @OnLifecycleEvent(Lifecycle.Event.ON_START)
    void onForeground() {
        EmbraceLogger.logDebug("AppState: App entered foreground.");
        isInBackground = false;
        StreamSupport.stream(listeners)
                .forEach(listener -> {
                    try {
                        listener.onForeground(coldStart, this.startTime);
                    } catch (Exception ex) {
                        EmbraceLogger.logDebug(ERROR_FAILED_TO_NOTIFY, ex);
                    }
                });
        coldStart = false;
    }

    /**
     * This method will be called by the ProcessLifecycleOwner when the main app process calls
     * ON STOP.
     **/
    @OnLifecycleEvent(Lifecycle.Event.ON_STOP)
    void onBackground() {
        EmbraceLogger.logDebug("AppState: App entered background");
        updateStateWithActivity(null);
        isInBackground = true;
        StreamSupport.stream(listeners).forEach(listener -> {
            try {
                listener.onBackground();
            } catch (Exception ex) {
                EmbraceLogger.logDebug(ERROR_FAILED_TO_NOTIFY, ex);
            }
        });
    }

    /**
     * Called when the OS has determined that it is a good time for a process to trim unneeded
     * memory.
     *
     * @param trimLevel the context of the trim, giving a hint of the amount of trimming.
     */
    @Override
    public void onTrimMemory(int trimLevel) {
        if (trimLevel == TRIM_MEMORY_RUNNING_LOW) {
            try {
                memoryService.onMemoryWarning();
            } catch (Exception ex) {
                EmbraceLogger.logDebug("Failed to handle onTrimMemory (low memory) event", ex);
            }
        }
    }

    @Override
    public void onConfigurationChanged(Configuration configuration) {

    }

    @Override
    public void onLowMemory() {

    }

    /**
     * Returns if the app's in background or not.
     */
    @Override
    public boolean isInBackground() {
        return isInBackground;
    }

    /**
     * Returns the current activity instance
     */
    @Override
    public Optional<Activity> getForegroundActivity() {
        Optional<Activity> foregroundActivity = Optional.fromNullable(currentActivity.get());
        if (!foregroundActivity.isPresent() ||
                foregroundActivity.get().isFinishing()) {
            return Optional.absent();
        }
        return foregroundActivity;
    }

    @Override
    public void addListener(ActivityListener listener, boolean priority) {
        if (!listeners.contains(listener)) {
            if (priority) {
                listeners.add(0, listener);
            } else {
                listeners.addIfAbsent(listener);
            }
        }
    }

    @Override
    public void addListener(ActivityListener listener) {
        addListener(listener, false);
    }

    @Override
    public void removeListener(ActivityListener listener) {
        listeners.remove(listener);
    }

    @Override
    public void close() {
        try {
            EmbraceLogger.logDebug("Shutting down EmbraceActivityService");
            application.getApplicationContext().unregisterComponentCallbacks(this);
            application.unregisterActivityLifecycleCallbacks(this);
            listeners.clear();
            appStateWorker.close();
        } catch (Exception ex) {
            EmbraceLogger.logDebug("Error when closing EmbraceActivityService", ex);
        }
    }
}

==> embrace-android-sdk-4.0.0-beta1-sources.jar: io/embrace/android/embracesdk/EmbraceActivityService.java <==
package io.embrace.android.embracesdk;

import android.app.Activity;
import android.app.Application;
import android.arch.lifecycle.Lifecycle;
import android.arch.lifecycle.OnLifecycleEvent;
import android.arch.lifecycle.ProcessLifecycleOwner;
import android.content.res.Configuration;
import android.os.Bundle;

import com.fernandocejas.arrow.checks.Preconditions;
import com.fernandocejas.arrow.optional.Optional;

import java.lang.ref.WeakReference;
import java.util.concurrent.CopyOnWriteArrayList;

import io.embrace.android.embracesdk.annotation.StartupActivity;
import java9.util.stream.StreamSupport;

/**
 * Service tracking the app's current activity and background state, and dispatching events to other
 * services as required.
 * <p>
 * See activity lifecycle documentation here:
 * https://developer.android.com/guide/components/activities/activity-lifecycle
 * <p>
 * We only need to track the `onForeground` and `onActivityPaused` lifecycle hooks for background
 * detection, as these are the only two which are consistently fired in all scenarios.
 */
final class EmbraceActivityService implements ActivityService {

    private static final String ERROR_FAILED_TO_NOTIFY = "Failed to notify EmbraceActivityService listener";
    /**
     * This Worker responsibility is to host the delayed execution of check to
     * establish is the app is in background or not.
     */
    private final ScheduledWorker appStateWorker = ScheduledWorker.ofSingleThread("App state service");

    /**
     * The memory service, it's provided on the instantiation of the service.
     */
    private final MemoryService memoryService;

    /**
     * The orientation service.
     */
    private volatile OrientationService orientationService;

    /**
     * List of listeners that subscribe to activity events.
     */
    private final CopyOnWriteArrayList<ActivityListener> listeners = new CopyOnWriteArrayList<>();

    /**
     * The application.
     */
    private final Application application;

    /**
     * The currently active activity.
     */
    private volatile WeakReference<Activity> currentActivity = new WeakReference<>(null);

    /**
     * States if the activity foreground phase comes from a cold start or not.
     * Checked every time an activity executes a foreground phase.
     */
    private volatile boolean coldStart = true;

    /**
     * States the initialization time of the EmbraceActivityService, inferring it is initialized
     * from the {@link Embrace#start(Application)} method.
     */
    private volatile long startTime;

    /**
     * Describes the application state.
     */
    private volatile boolean isInBackground = false;

    EmbraceActivityService(
            Application application,
            MemoryService memoryService,
            OrientationService orientationService) {

        this.memoryService = Preconditions.checkNotNull(memoryService);
        this.orientationService = Preconditions.checkNotNull(orientationService);
        this.application = Preconditions.checkNotNull(application);
        this.startTime = System.currentTimeMillis();
        application.registerActivityLifecycleCallbacks(this);
        application.getApplicationContext().registerComponentCallbacks(this);
        ProcessLifecycleOwner.get().getLifecycle().addObserver(this);
    }

    @Override
    public void onActivityCreated(Activity activity, Bundle bundle) {
        updateStateWithActivity(activity);
        updateOrientationWithActivity(activity);
    }

    @Override
    public void onActivityStarted(Activity activity) {
        updateStateWithActivity(activity);
        StreamSupport.stream(listeners).forEach(listener -> {
            try {
                listener.onView(activity);
            } catch (Exception ex) {
                EmbraceLogger.logDebug(ERROR_FAILED_TO_NOTIFY, ex);
            }
        });
    }

    @Override
    public void onActivityResumed(Activity activity) {
        if (!activity.getClass().isAnnotationPresent(StartupActivity.class)) {
            // If the activity coming to foreground doesn't have the StartupActivity annotation
            // the the SDK will finalize any pending startup moment.
            StreamSupport.stream(listeners).forEach(listener -> {
                try {
                    listener.applicationStartupComplete();
                } catch (Exception ex) {
                    EmbraceLogger.logDebug(ERROR_FAILED_TO_NOTIFY, ex);
                }
            });
        }
    }

    @Override
    public void onActivityPaused(Activity activity) {
    }

    @Override
    public void onActivityStopped(Activity activity) {
    }

    @Override
    public void onActivitySaveInstanceState(Activity activity, Bundle bundle) {
    }

    @Override
    public void onActivityDestroyed(Activity activity) {
    }

    /**
     * This method will update the current activity for further checking.
     *
     * @param activity the activity involved in the state change.
     */
    private synchronized void updateStateWithActivity(Activity activity) {
        this.currentActivity = new WeakReference<>(activity);
    }

    /**
     * This method will update the current activity orientation.
     *
     * @param activity the activity involved in the tracking orientation process.
     */
    private void updateOrientationWithActivity(Activity activity) {
        try {
            orientationService.onOrientationChanged(Optional.of(activity.getResources().getConfiguration().orientation));
        } catch (Exception ex) {
            EmbraceLogger.logDebug("Failed to register an orientation change", ex);
        }
    }

    /**
     * This method will be called by the ProcessLifecycleOwner when the main app process calls
     * ON START.
     **/
    @OnLifecycleEvent(Lifecycle.Event.ON_START)
    void onForeground() {
        EmbraceLogger.logDebug("AppState: App entered foreground.");
        isInBackground = false;
        StreamSupport.stream(listeners)
                .forEach(listener -> {
                    try {
                        listener.onForeground(coldStart, this.startTime);
                    } catch (Exception ex) {
                        EmbraceLogger.logDebug(ERROR_FAILED_TO_NOTIFY, ex);
                    }
                });
        coldStart = false;
    }

    /**
     * This method will be called by the ProcessLifecycleOwner when the main app process calls
     * ON STOP.
     **/
    @OnLifecycleEvent(Lifecycle.Event.ON_STOP)
    void onBackground() {
        EmbraceLogger.logDebug("AppState: App entered background");
        updateStateWithActivity(null);
        isInBackground = true;
        StreamSupport.stream(listeners).forEach(listener -> {
            try {
                listener.onBackground();
            } catch (Exception ex) {
                EmbraceLogger.logDebug(ERROR_FAILED_TO_NOTIFY, ex);
            }
        });
    }

    /**
     * Called when the OS has determined that it is a good time for a process to trim unneeded
     * memory.
     *
     * @param trimLevel the context of the trim, giving a hint of the amount of trimming.
     */
    @Override
    public void onTrimMemory(int trimLevel) {
        if (trimLevel == TRIM_MEMORY_RUNNING_LOW) {
            try {
                memoryService.onMemoryWarning();
            } catch (Exception ex) {
                EmbraceLogger.logDebug("Failed to handle onTrimMemory (low memory) event", ex);
            }
        }
    }

    @Override
    public void onConfigurationChanged(Configuration configuration) {
    }

    @Override
    public void onLowMemory() {
    }

    /**
     * Returns if the app's in background or not.
     */
    @Override
    public boolean isInBackground() {
        return isInBackground;
    }

    /**
     * Returns the current activity instance
     */
    @Override
    public Optional<Activity> getForegroundActivity() {
        Optional<Activity> foregroundActivity = Optional.fromNullable(currentActivity.get());
        if (!foregroundActivity.isPresent() ||
                foregroundActivity.get().isFinishing()) {
            return Optional.absent();
        }
        return foregroundActivity;
    }

    @Override
    public void addListener(ActivityListener listener, boolean priority) {
        if (!listeners.contains(listener)) {
            if (priority) {
                listeners.add(0, listener);
            } else {
                listeners.addIfAbsent(listener);
            }
        }
    }

    @Override
    public void addListener(ActivityListener listener) {
        addListener(listener, false);
    }

    @Override
    public void removeListener(ActivityListener listener) {
        listeners.remove(listener);
    }

    @Override
    public void close() {
        try {
            EmbraceLogger.logDebug("Shutting down EmbraceActivityService");
            application.getApplicationContext().unregisterComponentCallbacks(this);
            application.unregisterActivityLifecycleCallbacks(this);
            listeners.clear();
            appStateWorker.close();
        } catch (Exception ex) {
            EmbraceLogger.logDebug("Error when closing EmbraceActivityService", ex);
        }
    }
}
